﻿using System;
using System.Windows;
using System.Windows.Media;

namespace PolylineInterpolationDemo
{
    public class PointCollectionInterpolator : FrameworkElement
    {
        public static readonly DependencyProperty Points1Property =
            DependencyProperty.Register("Points1",
                typeof(PointCollection),
                typeof(PointCollectionInterpolator),
                new PropertyMetadata(OnPointsChanged));

        public static readonly DependencyProperty Points2Property =
            DependencyProperty.Register("Points2",
                typeof(PointCollection),
                typeof(PointCollectionInterpolator),
                new PropertyMetadata(OnPointsChanged));

        public static readonly DependencyProperty ProgressProperty =
            DependencyProperty.Register("Progress",
                typeof(double),
                typeof(PointCollectionInterpolator),
                new PropertyMetadata(0.0, OnProgressChanged));

        public static readonly DependencyProperty InterpolatedPointsProperty =
            DependencyProperty.Register("InterpolatedPoints",
                typeof(PointCollection),
                typeof(PointCollectionInterpolator),
                new PropertyMetadata(null));

        public PointCollectionInterpolator()
        {
            InterpolatedPoints = new PointCollection();
        }

        public PointCollection Points1
        {
            set { SetValue(Points1Property, value); }
            get { return (PointCollection)GetValue(Points1Property); }
        }

        public PointCollection Points2
        {
            set { SetValue(Points2Property, value); }
            get { return (PointCollection)GetValue(Points2Property); }
        }

        public double Progress
        {
            set { SetValue(ProgressProperty, value); }
            get { return (double)GetValue(ProgressProperty); }
        }

        public PointCollection InterpolatedPoints
        {
            set { SetValue(InterpolatedPointsProperty, value); }
            get { return (PointCollection)GetValue(InterpolatedPointsProperty); }
        }

        static void OnPointsChanged(DependencyObject obj, DependencyPropertyChangedEventArgs args)
        {
            (obj as PointCollectionInterpolator).OnChanged(args);
        }

        static void OnProgressChanged(DependencyObject obj, DependencyPropertyChangedEventArgs args)
        {
            (obj as PointCollectionInterpolator).OnChanged(args);
        }

        void OnChanged(DependencyPropertyChangedEventArgs args)
        {
            if (InterpolatedPoints == null || 
                    Points1 == null || Points2 == null)
                return;

            InterpolatedPoints.Clear();
            int num = Math.Min(Points1.Count, Points2.Count);

            for (int i = 0; i < num; i++)
                InterpolatedPoints.Add(
                    new Point((1 - Progress) * Points1[i].X + 
                                    Progress * Points2[i].X,
                              (1 - Progress) * Points1[i].Y + 
                                    Progress * Points2[i].Y));
        }
    }
}
